from __future__ import division, print_function

import random

import es
import events
import factory
import settings
import tools


class ExpirySystem(object):
    def __init__(self, master):
        self.master = master
        self.aspect = es.Aspect(master.world, "lifespan")

    def run(self):
        for entity in self.aspect:
            time = entity["lifespan"]
            time -= self.master.delta
            if time <= 0:
                if entity.has("death ticket"):
                    events.push(events.Event("ticket", entity["death ticket"]))
                entity.die()
            else:
                entity["lifespan"] = time

class DismissSystem(object):
    def __init__(self, master):
        self.master = master
        self.aspect = es.Aspect(master.world, "dismissable")

    def run(self):
        for entity in self.aspect:
            time = entity["dismissable"]
            time -= self.master.delta
            entity["dismissable"] = time
        for event in events.get():
            if event.kind == "key":
                for entity in self.aspect:
                    if entity["dismissable"] <= 0:
                        if entity.has("death ticket"):
                            events.push(events.Event("ticket", entity["death ticket"]))
                        entity.die()
            
class AiSystem(object):
    def __init__(self, master):
        self.master = master

    def run(self):
        player1, player2 = self.master.players
        if player1.ai:
            self.choose_card(player1, player2)
        elif player2.ai:
            self.choose_card(player2, player1)

    def choose_card(self, player, opponent):
        if not player.chosen_card:
            player.chosen_card = random.choice(player.hand)
            self.master.set_player_status(player, settings.READY_STATUS)


class ControlSystem(object):
    def __init__(self, master):
        self.master = master

    def run(self):
        players = self.master.players
        controls = settings.PLAYER_KEYS
        for event in events.get():
            if event.kind == "key":
                for i in xrange(2):
                    player = players[i]
                    if player.chosen_card:
                        continue
                    keys = controls[i]
                    key = event.value
                    if not player.ai and key in keys:
                        index = keys.index(key)
                        player.chosen_card = player.hand[index]
                        self.master.set_player_status(player, settings.READY_STATUS)
                        tools.play_sound("place.ogg")


class BattleSystem(object):
    def __init__(self, master):
        self.master = master
        self.battling = False
        self.finished = False

    def run(self):
        player1, player2 = self.master.players
        if self.battling or self.finished:
            return
        if player1.chosen_card and player2.chosen_card:
            self.centre_cards()
            ticket = factory.set_timer(settings.BATTLE_DELAY, self.master.world)
            events.schedule(events.Event("ticket", ticket), self.do_battle)
            self.battling = True

    def centre_cards(self):
        player1, player2 = self.master.players
        x,y = settings.DOCKET_POS
        player1.chosen_card["rect"].center = (x - 105, y-110)
        player2.chosen_card["rect"].center = (x + 105, y-110)

    def do_battle(self):
        player1, player2 = self.master.players
        winning_player = self.battle(player1, player2)
        rules = self.master.rules
        if winning_player:
            if rules.vs_cpu:
                if winning_player == player1:
                    tools.play_sound("win.ogg")
                elif winning_player == player2:
                    tools.play_sound("lose.ogg")
            else:
                if winning_player == player1:
                    tools.play_sound("win.ogg")
                elif winning_player == player2:
                    tools.play_sound("win2.ogg")
            if rules.use_chance:
                if winning_player.chance:
                    winning_player.points += 1
                else:
                    player1.chance = False
                    player2.chance = False
                    winning_player.chance = True
            else:
                winning_player.points += 1
        self.finish_round()
        if (winning_player and
            (winning_player.points >= settings.VICTORY_POINTS)):
                self.win_game(winning_player)

    def battle(self, player1, player2):
        card1 = player1.chosen_card["card"]
        card2 = player2.chosen_card["card"]
        if card1.beats(card2):
            self.master.alert("{} wins".format(player1.name), 100)
            return player1
        elif card2.beats(card1):
            self.master.alert("{} wins".format(player2.name), 100)
            return player2
        else:
            self.master.alert("Draw", 100)

    def finish_round(self):
        for player in self.master.players:
            player.hand.remove(player.chosen_card)
            self.master.deck.release_card(player.chosen_card["card"])
            player.chosen_card.die()
            player.chosen_card = None
            self.master.deal_card(player)
            self.master.set_player_status(player, settings.CHOOSE_STATUS)
        self.battling = False
        self.master.update_player_gui()

    def win_game(self, player):
        if self.master.rules.vs_match:
            self.master.finish_level()
        else:
            if player == self.master.players[0]:
                self.master.alert("Victory!", depth=60, colour=(100,255,100))
                ticket = factory.set_timer(1.5, self.master.world)
                events.schedule(events.Event("ticket", ticket),
                                self.master.finish_level)
            else:
                self.master.alert("Defeat...", depth=60, colour=(200,0,0))
                ticket = factory.set_timer(1.5, self.master.world)
                events.schedule(events.Event("ticket", ticket),
                                self.master.reset_level)
            self.finished = True
            for player in self.master.players:
                for card in player.hand:
                    card.die()
